@@ -203,9 +203,6 @@ UUBrokenByNetscape (char *string)
 
   ptr = string + len;
 
-  while (len && (*(ptr-1)=='\015' || *(ptr-1)=='\012')) {
-    ptr--; len--;
-  }
   if (len<3)         return 0;
   if (*--ptr == ' ') ptr--;
   ptr--;
@@ -297,13 +294,13 @@ int
 UUValidData (char *ptr, int encoding, int *bhflag)
 {
   int i=0, j, len=0, suspicious=0, flag=0;
-  char *s = ptr;
+  signed char *s = ptr;
 
   if ((s == NULL) || (*s == '\0')) {
     return 0;              /* bad string */
   }
 
-  while (*s && *s!='\012' && *s!='\015') {
+  while (*s) {
     s++;
     len++;
     i++;
@@ -545,8 +542,6 @@ UURepairData (FILE *datei, char *line, int encoding, int *bhflag)
       if (strlen (line) > 250)
 	break;
       ptr = line + strlen (line);
-      while (ptr>line && (*(ptr-1)=='\015' || *(ptr-1)=='\012'))
-	ptr--;
       if (_FP_fgets (ptr, 299-(ptr-line), datei) == NULL)
 	break;
     }
@@ -579,9 +574,6 @@ UURepairData (FILE *datei, char *line, int encoding, int *bhflag)
 
   if (vflag == 0) {
     ptr    = line + strlen(line);
-    while (ptr>line && (*(ptr-1)=='\012' || *(ptr-1)=='\015')) {
-      ptr--;
-    }
     *ptr++ = ' ';
     *ptr-- = '\0';
     if ((vflag = UUValidData (line, encoding, bhflag)) != UU_ENCODED) {
@@ -795,9 +787,9 @@ UUDecodeQP (FILE *datain, FILE *dataout, int *state,
 	p2 += 2;
 	p1  = p2;
       }
-      else if (*p2 == '\012' || *(p2+1) == '\015') {
+      else if (!*p2) {
 	/* soft line break */
-	*p2 = '\0';
+        goto skip_lbr;
 	break;
       }
       else {
@@ -811,7 +803,7 @@ UUDecodeQP (FILE *datain, FILE *dataout, int *state,
     val = 0;
     while (p2>p1 && isspace (*(p2-1))) {
       if (*(p2-1) == '\012' || *(p2-1) == '\015')
-	val = 1;
+        val = 1;
       p2--;
     }
     *p2 = '\0';
@@ -822,12 +814,15 @@ UUDecodeQP (FILE *datain, FILE *dataout, int *state,
      * encapsulation line is conceptually attached to the boundary.
      * So if the part ends here, don't print a line break"
      */
-    if (val && (!feof (datain) && 
-		(ftell(datain)<maxpos || flags&FL_TOEND ||
-		 (!(flags&FL_PROPER) && uu_fast_scanning))))
+    /* something is broken here now, but it was broken before */
+    if (!feof (datain) && 
+	(ftell(datain)<maxpos || flags&FL_TOEND ||
+	 (!(flags&FL_PROPER) && uu_fast_scanning)))
       fprintf (dataout, "%s\n", p1);
     else
       fprintf (dataout, "%s", p1);
+
+    skip_lbr: ;
   }
   return UURET_OK;
 }
@@ -875,18 +870,13 @@ UUDecodePT (FILE *datain, FILE *dataout, int *state,
 
     ptr = line + strlen (line);
 
-    while (ptr>line && (*(ptr-1) == '\012' || *(ptr-1) == '\015'))
-      ptr--;
-
-
     /*
      * If the part ends directly after this line, the data does not end
      * with a linebreak. Or, as the docs put it, "the CRLF preceding the
      * encapsulation line is conceptually attached to the boundary.
      * So if the part ends here, don't print a line break"
      */
-    if ((*ptr == '\012' || *ptr == '\015') &&
-	(ftell(datain)<maxpos || flags&FL_TOEND || flags&FL_PARTIAL ||
+    if ((ftell(datain)<maxpos || flags&FL_TOEND || flags&FL_PARTIAL ||
 	 !boundary || (!(flags&FL_PROPER) && uu_fast_scanning))) {
       *ptr = '\0';
       fprintf (dataout, "%s\n", line);
@@ -945,8 +935,8 @@ UUDecodeField (char *s, char *d, int method)
 	  count++;
 	  s+=3;
 	}
-	else if (*(s+1) == '\012' || *(s+1) == '\015') {
-	  s+=2;
+	else if (!s[1]) {
+          d[count++] = '\012';
 	}
 	else {
 	  d[count++] = *s++;
@@ -970,7 +960,7 @@ UUDecodePart (FILE *datain, FILE *dataout, int *state,
   char *line, *oline=uuncdp_oline;
   int warning=0, vlc=0, lc[2], hadct=0;
   int tc=0, tf=0, vflag, haddata=0, haddh=0;
-  long yefilesize=0, yepartends=0;
+  long yefilesize=0, yepartends=0, yenotlastpart=0;
   crc32_t yepartcrc=crc32(0L, Z_NULL, 0);
   static crc32_t yefilecrc=0;
   static int bhflag=0;
@@ -1021,7 +1011,7 @@ UUDecodePart (FILE *datain, FILE *dataout, int *state,
       return UURET_IOERR;
     }
 
-    if (line[0]=='\015' || line[0]=='\012') { /* Empty line? */
+    if (!*line) { /* Empty line? */
       if (*state == DATA &&
 	  (method == UU_ENCODED || method == XX_ENCODED))
 	*state = END;
@@ -1119,7 +1109,12 @@ UUDecodePart (FILE *datain, FILE *dataout, int *state,
 	  yefilesize = -1;
 	}
 
-	if (_FP_strstr (line, " part=") != NULL) {
+	if ((ptr =_FP_strstr (line, " part="))) {
+          int partno = atoi (ptr + 6);
+       
+	  if ((ptr = _FP_strstr (line, " total=")))
+            yenotlastpart = atoi (ptr + 7) != partno;
+
 	  if (_FP_fgets (line, 1200 - 5, datain) == NULL) {
 	    break;
 	  }
@@ -1127,7 +1122,7 @@ UUDecodePart (FILE *datain, FILE *dataout, int *state,
 	  if ((ptr = _FP_strstr (line, " end=")) == NULL) {
 	    break;
 	  }
-       
+
 	  yepartends = atoi (ptr + 5);
 	}
 	tf = 1;
@@ -1140,7 +1135,7 @@ UUDecodePart (FILE *datain, FILE *dataout, int *state,
       tc = tf = vlc = 0;
       lc[0] = lc[1] = 0;
     }
-    else if ((*state == END) &&
+    else if ((*state == END || *state == DATA) &&
 	     (method == UU_ENCODED || method == XX_ENCODED)) {
       if (strncmp (line, "end", 3) == 0) {
 	*state = DONE;
@@ -1179,7 +1174,7 @@ UUDecodePart (FILE *datain, FILE *dataout, int *state,
 		       yepartsize, size);
 	}
       }
-      if (yepartends == 0 || yepartends >= yefilesize) {
+      if (!yenotlastpart && (yepartends == 0 || yepartends >= yefilesize)) {
 	*state = DONE;
       }
       break;
@@ -1207,8 +1202,7 @@ UUDecodePart (FILE *datain, FILE *dataout, int *state,
 	if (tf) {
 	  count  = UUDecodeLine (line, oline, method);
 	  if (method == YENC_ENCODED) {
-	    if (yepartends)
-	      yepartcrc = crc32(yepartcrc, oline, count);
+	    yepartcrc = crc32(yepartcrc, oline, count);
 	    yefilecrc = crc32(yefilecrc, oline, count);
 	    yepartsize += count;
 	  }
@@ -1312,8 +1306,9 @@ int
 UUDecode (uulist *data)
 {
   int state=BEGIN, part=-1, res=0, hb;
-  long rsize, dsize, numbytes;
+  unsigned long rsize, dsize, numbytes;
   FILE *datain, *dataout;
+  void *datain_buf, *dataout_buf;
   unsigned char r[8];
   char *mode, *ntmp;
   uufile *iter;
@@ -1388,6 +1383,7 @@ UUDecode (uulist *data)
     uu_errno = errno;
     return UURET_IOERR;
   }
+  UUSETBUF (dataout, dataout_buf, uu_wbuf);
 
   /*
    * we don't have begin lines in Base64 or plain text files.
@@ -1480,6 +1476,7 @@ UUDecode (uulist *data)
       }
       _FP_strncpy (uugen_fnbuffer, iter->data->sfname, 1024);
     }
+    UUSETBUF (datain, datain_buf, uu_rbuf);
 
     progress.partno  = part;
     progress.fsize   = (iter->data->length)?iter->data->length:-1;
@@ -1491,6 +1488,7 @@ UUDecode (uulist *data)
 			iter->data->startpos+iter->data->length,
 			data->uudet, iter->data->flags, NULL);
     fclose             (datain);
+    UUCLRBUF (uu_rbuf, datain_buf);
 
     if (uu_FileCallback)
       (*uu_FileCallback) (uu_FileCBArg, iter->data->sfname, uugen_fnbuffer, 0);
@@ -1512,6 +1510,7 @@ UUDecode (uulist *data)
 	       strerror (uu_errno = errno));
     res = UURET_IOERR;
   }
+  UUCLRBUF (uu_wbuf, dataout_buf);
 
   if (res != UURET_OK || (state != DONE && !uu_desperate)) {
     unlink (data->binfile);
@@ -1557,10 +1556,12 @@ UUDecode (uulist *data)
       free (ntmp);
       return UURET_IOERR;
     }
+    UUSETBUF (datain, datain_buf, uu_rbuf);
+
 #ifdef HAVE_MKSTEMP
-	strcpy(ntmp, tmpdir);
-	strcat(ntmp, "/");
-	strcat(ntmp, tmpprefix); 
+    strcpy(ntmp, tmpdir);
+    strcat(ntmp, "/");
+    strcat(ntmp, tmpprefix); 
     if ((tmpfd = mkstemp(ntmp)) == -1 ||
 		(dataout = fdopen(tmpfd, "wb")) == NULL) {
 #else
@@ -1571,6 +1572,7 @@ UUDecode (uulist *data)
 		 ntmp, strerror (uu_errno = errno));
       progress.action = 0;
       fclose (datain);
+      UUCLRBUF (uu_rbuf, datain_buf);
 #ifdef HAVE_MKSTEMP
 	  if (tmpfd != -1) {
 		  unlink(ntmp);
@@ -1580,11 +1582,12 @@ UUDecode (uulist *data)
       free   (ntmp);
       return UURET_IOERR;
     }
+    UUSETBUF (dataout, dataout_buf, uu_wbuf);
 
     /*
      * read fork lengths. remember they're in Motorola format
      */
-    r[0] = fgetc (datain);
+    r[0] = _FP_fgetc (datain);
     hb   = (int) r[0] + 22;
     fseek (datain, (int) r[0] + 12, SEEK_SET);
     fread (r, 1, 8, datain);
@@ -1621,7 +1624,7 @@ UUDecode (uulist *data)
     progress.action   = 0;
     progress.partno   = 0;
     progress.numparts = 1;
-    progress.fsize    = (numbytes)?numbytes:-1;
+    progress.fsize    = numbytes ? numbytes : -1;
     progress.foffset  = hb;
     progress.percent  = 0;
     progress.action   = UUACT_COPYING;
@@ -1635,7 +1638,9 @@ UUDecode (uulist *data)
 	UUMessage (uunconc_id, __LINE__, UUMSG_NOTE,
 		   uustring (S_DECODE_CANCEL));
 	fclose (datain);
+        UUCLRBUF (uu_rbuf, datain_buf);
 	fclose (dataout);
+        UUCLRBUF (uu_wbuf, dataout_buf);
 	unlink (ntmp);
 	free   (ntmp);
 	return UURET_CANCEL;
@@ -1650,7 +1655,9 @@ UUDecode (uulist *data)
 		   uustring (S_SOURCE_READ_ERR),
 		   data->binfile, strerror (uu_errno = errno));
 	fclose (datain);
+        UUCLRBUF (uu_rbuf, datain_buf);
 	fclose (dataout);
+        UUCLRBUF (uu_wbuf, dataout_buf);
 	unlink (ntmp);
 	free   (ntmp);
 	return UURET_IOERR;
@@ -1661,7 +1668,9 @@ UUDecode (uulist *data)
 		   uustring (S_WR_ERR_TARGET),
 		   ntmp, strerror (uu_errno = errno));
 	fclose (datain);
+        UUCLRBUF (uu_rbuf, datain_buf);
 	fclose (dataout);
+        UUCLRBUF (uu_wbuf, dataout_buf);
 	unlink (ntmp);
 	free   (ntmp);
 	return UURET_IOERR;
@@ -1682,7 +1691,9 @@ UUDecode (uulist *data)
      */
 
     fclose (datain);
+    UUCLRBUF (uu_rbuf, datain_buf);
     if (fclose (dataout)) {
+      UUCLRBUF (uu_wbuf, dataout_buf);
       UUMessage (uunconc_id, __LINE__, UUMSG_ERROR,
 		 uustring (S_WR_ERR_TARGET),
 		 ntmp, strerror (uu_errno = errno));
@@ -1690,6 +1701,7 @@ UUDecode (uulist *data)
       free   (ntmp);
       return UURET_IOERR;
     }
+    UUCLRBUF (uu_wbuf, dataout_buf);
 
     if (unlink (data->binfile)) {
       UUMessage (uunconc_id, __LINE__, UUMSG_WARNING,
